#include "StdAfx.h"
#include "leohelpers.h"

///

#define USER32DLL_NAME (_T("user32.dll"))
#define MONITORFROMPOINT_NAME ("MonitorFromPoint")
#define MONITORFROMWINDOW_NAME ("MonitorFromWindow")
#ifdef UNICODE
#define GETMONITORINFO_NAME ("GetMonitorInfoW")
#else
#define GETMONITORINFO_NAME ("GetMonitorInfoA")
#endif // !UNICODE

typedef WINUSERAPI HMONITOR (WINAPI *MONITORFROMPOINT_PTR)(IN POINT pt, IN DWORD dwFlags);
typedef WINUSERAPI BOOL (WINAPI *GETMONITORINFO_PTR)(IN HMONITOR hMonitor, OUT LPMONITORINFO lpmi);
typedef WINUSERAPI HMONITOR (WINAPI *MONITORFROMWINDOW_PTR)(IN HWND hwnd, IN DWORD dwFlags);

///

bool NLeoHelpers::LeetRegQueryValue(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szValueName,
									   DWORD *pdwType, void **plpData, DWORD *pcbData)
{
	// Documentation doesn't state that registry functions call SetLastError().
	// We need to preserve the error values they return through clean-up anyway.
	LONG regRes = ERROR_SUCCESS;

	HKEY hKeyToReadValueFrom;

	DWORD dwSomewhatDamaged; // Used for getting the registry size if the user doesn't care about it.

	if (NULL == pcbData)
	{
		pcbData = &dwSomewhatDamaged;
	}

	// szKeyPath may be NULL in which case another handle like hKeyParent is openned (and must be closed).
	regRes = RegOpenKeyEx(hKeyParent, szKeyPath, 0, KEY_QUERY_VALUE, &hKeyToReadValueFrom);

	if (ERROR_SUCCESS == regRes)
	{
		if (NULL == plpData)
		{
			// They just want the type and/or the data size.
			regRes = RegQueryValueEx(hKeyToReadValueFrom, szValueName, 0, pdwType, NULL, pcbData);
		}
		else
		{
			BYTE *allocatedBuffer = NULL;
			DWORD dwAllocatedBufferSize = 0; // For HKEY_PERFORMANCE_DATA, really.

			// Get the required buffer size by giving NULL for data buffer.
			regRes = RegQueryValueEx(hKeyToReadValueFrom, szValueName, 0, pdwType, NULL, pcbData);

			if (ERROR_SUCCESS == regRes)
			{
				dwAllocatedBufferSize = *pcbData;
				allocatedBuffer = new BYTE[dwAllocatedBufferSize];
			}
			else if (ERROR_MORE_DATA == regRes)
			{
				// Most likely tried to read HKEY_PERFORMANCE_DATA.
				dwAllocatedBufferSize = 1024; // Must be a number x such that (Int(x/2) > 0)
				allocatedBuffer = new BYTE[dwAllocatedBufferSize];
			}
			// else leave allocatedBuffer NULL so the while loop does not execute.

			while (NULL != allocatedBuffer)
			{
				regRes = RegQueryValueEx(hKeyToReadValueFrom, szValueName, 0, pdwType, allocatedBuffer, pcbData);

				if (ERROR_SUCCESS == regRes)
				{
					*plpData = allocatedBuffer;
					allocatedBuffer = NULL;
					break;
				}
				else if (ERROR_MORE_DATA == regRes)
				{
					// Try a larger buffer.
					delete [] allocatedBuffer;
					dwAllocatedBufferSize = dwAllocatedBufferSize + (dwAllocatedBufferSize/2);
					allocatedBuffer = new BYTE[dwAllocatedBufferSize];
				}
				else
				{
					delete [] allocatedBuffer;
					allocatedBuffer = NULL;
					break;
				}
			}
		}

		RegCloseKey(hKeyToReadValueFrom);
	}

	if (ERROR_SUCCESS != regRes)
	{
		SetLastError(regRes);
	}

	return(ERROR_SUCCESS == regRes);
}

bool NLeoHelpers::LeetRegQueryDWORDValue(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szValueName, DWORD *pdwRes)
{
	DWORD dwErrToSet = ERROR_SUCCESS;

	DWORD dwType;
	void *pData;

	if (NULL != pdwRes)
	{
		*pdwRes = 0;
	}

	if (! LeetRegQueryValue(hKeyParent, szKeyPath, szValueName, &dwType, &pData, NULL) )
	{
		dwErrToSet = GetLastError();
	}
	else
	{
		if (REG_DWORD != dwType)
		{
			dwErrToSet = ERROR_INVALID_DATA;
		}
		else
		{
			if (NULL != pdwRes)
			{
				*pdwRes = *((DWORD*)pData);
			}
		}

		delete [] pData;
	}

	if (ERROR_SUCCESS != dwErrToSet)
	{
		SetLastError(dwErrToSet);
	}

	return(ERROR_SUCCESS == dwErrToSet);
}

// Returns false on failure. Use GetLastError().
// Uses LeetRegQueryValue() to get you an int without having to worry about buffers, types and so on.
bool NLeoHelpers::LeetRegQueryIntValue(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szValueName, int *pInt)
{
	DWORD dwData = 0;

	if (!LeetRegQueryDWORDValue(hKeyParent, szKeyPath, szValueName, &dwData))
	{
		return false;
	}

	if (NULL != pInt && sizeof(int)==sizeof(DWORD))
	{
		*pInt = *reinterpret_cast<int*>(&dwData);
	}

	return true;
}

// Returns false on failure. Use GetLastError().
// Uses LeetRegQueryValue() to get you a bool without having to worry about buffers, types and so on.
bool NLeoHelpers::LeetRegQueryBoolValue(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szValueName, bool *pBool)
{
	DWORD dwData = 0;

	if (!LeetRegQueryDWORDValue(hKeyParent, szKeyPath, szValueName, &dwData))
	{
		return false;
	}

	if (NULL != pBool)
	{
		*pBool = (dwData ? true : false);
	}

	return true;
}

// Returns false on failure. Use GetLastError().
// Uses LeetRegQueryValue() to get you a TCHAR string without having to worry about buffers, types and so on.
bool NLeoHelpers::LeetRegQueryStringValue(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szValueName, std::basic_string<TCHAR> *pString)
{
	DWORD dwErrToSet = ERROR_SUCCESS;

	DWORD dwType;
	void *pData;

	pString->erase();

	if (! LeetRegQueryValue(hKeyParent, szKeyPath, szValueName, &dwType, &pData, NULL) )
	{
		dwErrToSet = GetLastError();
	}
	else
	{
		if (REG_SZ != dwType)
		{
			dwErrToSet = ERROR_INVALID_DATA;
		}
		else
		{
			(*pString) = reinterpret_cast<TCHAR*>(pData);
		}

		delete [] pData;
	}

	if (ERROR_SUCCESS != dwErrToSet)
	{
		SetLastError(dwErrToSet);
	}

	return(ERROR_SUCCESS == dwErrToSet);
}

bool NLeoHelpers::LeetRegQueryMultiStringValue(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szValueName, std::vector< std::basic_string<TCHAR> > *pVecStrings)
{
	DWORD dwErrToSet = ERROR_SUCCESS;

	DWORD dwType;
	void *pData;

	pVecStrings->clear();

	if (! LeetRegQueryValue(hKeyParent, szKeyPath, szValueName, &dwType, &pData, NULL) )
	{
		dwErrToSet = GetLastError();
	}
	else
	{
		if (REG_MULTI_SZ != dwType)
		{
			dwErrToSet = ERROR_INVALID_DATA;
		}
		else
		{
			const TCHAR *msz = reinterpret_cast<TCHAR*>(pData);

			for ( std::basic_string<TCHAR> strLine = msz; 0 < strLine.length(); strLine = ( msz = (msz + strLine.length() + 1) ) )
			{
				pVecStrings->push_back( strLine );
			}
		}

		delete [] pData;
	}

	if (ERROR_SUCCESS != dwErrToSet)
	{
		SetLastError(dwErrToSet);
	}

	return(ERROR_SUCCESS == dwErrToSet);
}

bool NLeoHelpers::LeetRegSetValue(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szValueName,
									 DWORD dwType, const void *lpData, DWORD cbData)
{
	// Documentation doesn't state that registry functions call SetLastError().
	// We need to preserve the error values they return through clean-up anyway.
	LONG regRes = ERROR_SUCCESS;

	bool bCloseKey = false;
	HKEY hKeyToCreateValueIn;

	if ((NULL == szKeyPath) || (0 == _tcsicmp(szKeyPath, _T(""))))
	{
		// Cannot give NULL as second argument to RegCreateKeyEx() as you can to RegOpenKeyEx()
		hKeyToCreateValueIn = hKeyParent;
	}
	else
	{
		DWORD disposition;

		// We don't need to create the key one level at a time, RegCreateKeyEx() does that.
		regRes = RegCreateKeyEx(hKeyParent, szKeyPath, 0, _T(""),
								REG_OPTION_NON_VOLATILE, KEY_SET_VALUE, NULL,
								&hKeyToCreateValueIn, &disposition);

		if (ERROR_SUCCESS == regRes)
		{
			bCloseKey = true;
		}
	}

	if ((ERROR_SUCCESS == regRes) && (NULL != lpData))
	{
		regRes = RegSetValueEx(hKeyToCreateValueIn, szValueName, 0, dwType, (CONST BYTE *)lpData, cbData);
	}

	if (bCloseKey)
	{
		RegCloseKey(hKeyToCreateValueIn);
		bCloseKey = false;
	}

	if (ERROR_SUCCESS != regRes)
	{
		SetLastError(regRes);
	}

	return(ERROR_SUCCESS == regRes);
}

bool NLeoHelpers::LeetRegSetDWORDValue(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szValueName, DWORD dwData)
{
	return(LeetRegSetValue(hKeyParent, szKeyPath, szValueName, REG_DWORD, &dwData, sizeof(DWORD)));
}

bool NLeoHelpers::LeetRegSetIntValue(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szValueName, int iData)
{
	if (sizeof(int)==sizeof(DWORD))
	{
		DWORD dwData = *reinterpret_cast<DWORD*>(&iData);

		return(LeetRegSetValue(hKeyParent, szKeyPath, szValueName, REG_DWORD, &dwData, sizeof(DWORD)));
	}

	return false;
}

bool NLeoHelpers::LeetRegSetBoolValue(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szValueName, bool bData)
{
	DWORD dwData = bData ? 1 : 0;

	return(LeetRegSetValue(hKeyParent, szKeyPath, szValueName, REG_DWORD, &dwData, sizeof(DWORD)));
}

bool NLeoHelpers::LeetRegSetStringValue(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szValueName, const TCHAR *szData)
{
	return(LeetRegSetValue(hKeyParent, szKeyPath, szValueName, REG_SZ, szData, (_tcslen(szData)+1)*sizeof(szData[0])));
}

bool NLeoHelpers::LeetRegSetMultiStringValue(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szValueName, const std::vector< std::basic_string<TCHAR> > *pVecStrings)
{
	bool bResult = false;

	DWORD dwBufferSize = 1; // Final null character.

	for (std::vector< std::basic_string<TCHAR> >::const_iterator pStr = pVecStrings->begin(); pStr != pVecStrings->end(); pStr++)
	{
		if (!pStr->empty())
		{
			dwBufferSize += (pStr->length() + 1);
		}
	}

	TCHAR *mszBuffer = new TCHAR[ dwBufferSize ];

	if (NULL != mszBuffer)
	{
		TCHAR *szCurrent = mszBuffer;

		for (std::vector< std::basic_string<TCHAR> >::const_iterator pStr = pVecStrings->begin(); pStr != pVecStrings->end(); pStr++)
		{
			if (!pStr->empty())
			{
				_tcscpy(szCurrent, pStr->c_str());

				szCurrent += (pStr->length() + 1);
			}
		}

		szCurrent[0] = _T('\0'); // Final null character.

		bResult = LeetRegSetValue(hKeyParent, szKeyPath, szValueName, REG_MULTI_SZ, mszBuffer, dwBufferSize);

		delete [] mszBuffer;
	}

	return(bResult);
}

bool NLeoHelpers::LeetRegEnumKey(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szPrefixFilter, std::vector< std::basic_string<TCHAR> > *pVecStrings)
{
	// Documentation doesn't state that registry functions call SetLastError().
	// We need to preserve the error values they return through clean-up anyway.
	LONG regRes = ERROR_SUCCESS;

	pVecStrings->clear();

	HKEY hKeyToEnum;

	// szKeyPath may be NULL in which case another handle like hKeyParent is openned (and must be closed).
	regRes = RegOpenKeyEx(hKeyParent, szKeyPath, 0, KEY_ENUMERATE_SUB_KEYS, &hKeyToEnum);

	if (ERROR_SUCCESS == regRes)
	{
		FILETIME ftLastWriteTime;	// Don't care but documentation doesn't say it can be null.

		TCHAR szNameBuffer[1024];
		const DWORD dwNameBufferSize = (sizeof(szNameBuffer)/sizeof(szNameBuffer[0]));
		DWORD dwcName;

		size_t cPrefixFilter = (NULL != szPrefixFilter) ? strlen(szPrefixFilter) : 0;

		for(DWORD dwIndex = 0; true; dwIndex++)
		{
			dwcName = dwNameBufferSize;
			regRes = RegEnumKeyEx(hKeyToEnum, dwIndex, szNameBuffer, &dwcName, NULL, NULL, NULL, &ftLastWriteTime);

			if (ERROR_SUCCESS == regRes && dwcName >= dwNameBufferSize)
			{
				regRes = ERROR_INSUFFICIENT_BUFFER;
				break;
			}
			else if (ERROR_NO_MORE_ITEMS == regRes)
			{
				regRes = ERROR_SUCCESS;
				break;
			}
			else if (ERROR_SUCCESS != regRes)
			{
				break;
			}
			else
			{
				if (0 == cPrefixFilter || 0 == _strnicmp(szNameBuffer, szPrefixFilter, cPrefixFilter))
				{
					pVecStrings->push_back( szNameBuffer + cPrefixFilter );
				}
			}
		}

		RegCloseKey(hKeyToEnum);
	}

	if (ERROR_SUCCESS != regRes)
	{
		SetLastError(regRes);
	}

	return(ERROR_SUCCESS == regRes);
}

bool NLeoHelpers::LeetRegDeleteKey(HKEY hKeyParent, const TCHAR *szKeyPath, const TCHAR *szKeyToDelete)
{
	// Documentation doesn't state that registry functions call SetLastError().
	// We need to preserve the error values they return through clean-up anyway.
	LONG regRes = ERROR_SUCCESS;

	HKEY hKeyToDeleteUnder;

	// szKeyPath may be NULL in which case another handle like hKeyParent is openned (and must be closed).
	regRes = RegOpenKeyEx(hKeyParent, szKeyPath, 0, DELETE, &hKeyToDeleteUnder);

	if (ERROR_SUCCESS == regRes)
	{
		regRes = RegDeleteKey(hKeyToDeleteUnder, szKeyToDelete);

		RegCloseKey(hKeyToDeleteUnder);
	}

	if (ERROR_SUCCESS != regRes)
	{
		SetLastError(regRes);
	}

	return(ERROR_SUCCESS == regRes);
}

TCHAR *NLeoHelpers::GetParentPath(const TCHAR *path)
{
	TCHAR *result = NULL;

	if (NULL != path)
	{
		const TCHAR *lastBackslash  = _tcsrchr(path, _T('\\'));
		const TCHAR *lastFrontslash = _tcsrchr(path, _T('/') );
		const TCHAR *lastSlash = NULL;

		if ((NULL != lastBackslash) && (NULL != lastFrontslash))
		{
			if (lastBackslash > lastFrontslash)
			{
				lastSlash = lastBackslash;
			}
			else
			{
				lastSlash = lastFrontslash;
			}
		}
		else if (NULL != lastBackslash)
		{
			lastSlash = lastBackslash;
		}
		else if (NULL != lastFrontslash)
		{
			lastSlash = lastFrontslash;
		}

		if (NULL == lastSlash)
		{
			result = new TCHAR[1];
			result[0] = _T('\0');
		}
		else
		{
			size_t resSize = (lastSlash - path) + 1;
			result = new TCHAR[resSize];
			NLeoHelpers::StringCopy(result, path, resSize);
		}
	}

	return(result);
}

int NLeoHelpers::StringCopy(TCHAR *dest, const TCHAR *source, int destSizeChars)
{
	int idx = 0;

	if (destSizeChars > 0)
	{
		while ((_T('\0') != source[idx]) && (destSizeChars > 1))
		{
			dest[idx] = source[idx];
			idx++;
			destSizeChars--;
		}

		dest[idx] = _T('\0');
	}

	return(idx);
}

// _snprintf doesn't always null term the destination... sigh
// Returns the index of the null terminator.
int NLeoHelpers::StringFormat(TCHAR *buffer, size_t bufferSizeChars, const TCHAR *format, ...)
{
	va_list args;
	va_start(args, format);
	int result = VStringFormat(buffer, bufferSizeChars, format, args);
	va_end(args);

	return(result);
}

// _vsnprintf doesn't always null term the destination... sigh
// Returns the index of the null terminator.
int NLeoHelpers::VStringFormat(TCHAR *buffer, size_t bufferSizeChars, const TCHAR *format, va_list args)
{
	int result = 0;

	if (bufferSizeChars > 0)
	{
		result = _vsntprintf(buffer, bufferSizeChars, format, args);

		if (0 > result)
		{
			result = bufferSizeChars - 1; // LD 20/Mar/2000
			buffer[result] = _T('\0');
		}
	}

	return(result);
}

// delete[] the result.
// Returns NULL on failure.
TCHAR *NLeoHelpers::StringAllocAndFormat(const TCHAR *format, ...)
{
	va_list args;
	va_start(args, format);
	TCHAR *result = VStringAllocAndFormat(format, args);
	va_end(args);

	return(result);
}

// delete[] the result.
// Returns NULL on failure.
TCHAR *NLeoHelpers::VStringAllocAndFormat(const TCHAR *format, va_list args)
{
	size_t bufferSize = 1024;

	TCHAR *szResult = new TCHAR[bufferSize];

	while(true)
	{
		int formatRes = _vsntprintf(szResult, bufferSize, format, args);

		// formatRes negative on error; (bufferSize == formatRes) should be redundant for _vsntprintf.

		if ((0 > formatRes) || (bufferSize == formatRes))
		{
			delete [] szResult;
			bufferSize *= 2;
			szResult = new TCHAR[bufferSize];
		}
		else
		{
			break;
		}
	}

	return(szResult);
}

TCHAR *NLeoHelpers::JoinPaths(const TCHAR *first, const TCHAR *second)
{
	TCHAR *result = NULL;

	int firstLen = _tcslen(first);
	int length = firstLen + _tcslen(second) + 1;

	// LD 10/Jan/2002 Now checks that the start of second isn't a slash either.

	bool addSlash = ((firstLen > 0)			&& (_T('\\') != first[firstLen - 1]) && (_T('/') != first[firstLen - 1])\
				&&	 (_tcslen(second) > 0)	&& (_T('\\') != second[0])			 && (_T('/') != second[0]));

	if (addSlash)
	{
		length++; // Space for the extra backslash.
	}

	result = new TCHAR[length];

	_tcscpy(result, first);

	if (addSlash)
	{
		result[firstLen] = _T('\\');
		_tcscpy(result + firstLen + 1, second);
	}
	else
	{
		_tcscpy(result + firstLen, second);
	}

	return(result);
}

HANDLE NLeoHelpers::OpenTempFileNamePreserveExtension(const TCHAR *szTempPath, const TCHAR *szPrefix, const TCHAR *szFilenameIn,
													  std::basic_string< TCHAR > *pstrFilenameOut)
{
	HANDLE hFileResult = INVALID_HANDLE_VALUE;

	std::basic_string< TCHAR > strFileStart = szTempPath;

	if (strFileStart.length() > 0 && '\\' != *(strFileStart.rbegin()) && '/' != *(strFileStart.rbegin()))
	{
		strFileStart += '\\';
	}

	strFileStart += szPrefix;

	std::basic_string< TCHAR > strExtension = ".tmp";

	if (NULL != szFilenameIn)
	{
		const TCHAR *szExt = strrchr(szFilenameIn, '.');

		if (NULL != szExt)
		{
			strExtension = szExt;
		}
	}

	SYSTEMTIME systime;
	GetSystemTime(&systime);

	int i1 = systime.wYear;
	int i2 = systime.wMonth;
	int i3 = systime.wDay;
	int i4 = systime.wHour;
	int i5 = systime.wMinute;
	int i6 = systime.wSecond;
	int i7a = systime.wMilliseconds;

	for (int i = 0; i < 100; i++)
	{
		int i7 = i7a + i;

		TCHAR *szFilename = StringAllocAndFormat(_T("%s%04d%02d%02d%02d%02d%02d%04d%s"),
													strFileStart.c_str(), i1,i2,i3,i4,i5,i6,i7, strExtension.c_str());

		if (NULL == szFilename)
		{
			break;
		}
		else
		{
			hFileResult = CreateFile(szFilename, GENERIC_WRITE, 0, 0, CREATE_NEW, FILE_ATTRIBUTE_NORMAL, 0);

			if (hFileResult != INVALID_HANDLE_VALUE)
			{
				if (NULL != pstrFilenameOut)
				{
					*pstrFilenameOut = szFilename;
				}
				delete [] szFilename;
				break;
			}
			delete [] szFilename;
		}
	}

	return(hFileResult);
}

std::string NLeoHelpers::GenerateSafeName(const char *szInput)
{
	std::string strResult;

	if (NULL != szInput)
	{
		while(*szInput)
		{
			if (isalpha(*szInput) || isdigit(*szInput))
			{
				strResult += *szInput++;
			}
			else
			{
				strResult += "_";
				szInput++;
			}
		}
	}

	return strResult;
}

WCHAR *NLeoHelpers::LeetMBtoWC(const char *mbString)
{
	WCHAR *result = NULL;

	int wcBufSize = MultiByteToWideChar(CP_ACP, 0, mbString, -1, NULL, 0);

	result = new WCHAR[wcBufSize];

	// Hopefully this includes the '\0' in the copy and the count of characters,
	// else this will not work with empty strings.
	if (0 == MultiByteToWideChar(CP_ACP, 0, mbString, -1, result, wcBufSize))
	{
		delete [] result;
		result = NULL;
	}

	return(result);
}

char *NLeoHelpers::LeetWCtoMB(const WCHAR *wcString)
{
	char *result = NULL;

	int mbBufSize = WideCharToMultiByte(CP_ACP, 0, wcString, -1, NULL, 0, NULL, NULL);

	result = new char[mbBufSize];

	// Hopefully this includes the '\0' in the copy and the count of characters,
	// else this will not work with empty strings.
	if (0 == WideCharToMultiByte(CP_ACP, 0, wcString, -1, result, mbBufSize, NULL, NULL))
	{
		delete [] result;
		result = NULL;
	}

	return(result);
}

const TCHAR *NLeoHelpers::GetLastPathPart(const TCHAR *szIn)
{
	if (NULL == szIn)
	{
		return(szIn);
	}

	size_t len = _tcslen(szIn);

	if (1 >= len)
	{
		return(szIn);
	}

	const TCHAR *szOut = szIn + (len - 1);

	// Ignore any slashes at the very end of the string.
	while (szIn < szOut && (*szOut == _T('\\') || *szOut == _T('/')))
	{
		szOut--;
	}

	// Stop at the next slash we find.
	while (szIn < szOut && (*szOut != _T('\\') && *szOut != _T('/')))
	{
		szOut--;
	}

	if (*szOut == _T('\\') || *szOut == _T('/'))
	{
		szOut++;
	}

	return(szOut);
}

BOOL NLeoHelpers::ScreenToClientRect(HWND hwnd, RECT *pRect)
{
	BOOL bResult = FALSE;

	POINT ptTL;
	POINT ptBR;

	ptTL.x = pRect->left;
	ptTL.y = pRect->top;
	ptBR.x = pRect->right;
	ptBR.y = pRect->bottom;

	if (ScreenToClient(hwnd, &ptTL) && ScreenToClient(hwnd, &ptBR))
	{
		pRect->left   = ptTL.x;
		pRect->top    = ptTL.y;
		pRect->right  = ptBR.x;
		pRect->bottom = ptBR.y;

		bResult = TRUE;
	}

	return(bResult);
}

/*
BOOL NLeoHelpers::OffsetDlgItem(HWND hwndDlg, int nIDDlgItem, LONG dx, LONG dy, LONG dw, LONG dh, BOOL bRepaint)
{
	BOOL bResult = FALSE;
	HWND hwnd = hwndDlg; // We'll adjust the dialog itself if nIDDlgItem is zero.

	if (0 != nIDDlgItem && NULL != hwndDlg)
	{
		hwnd = GetDlgItem(hwndDlg, nIDDlgItem);
	}

	if (NULL != hwnd)
	{
		RECT r;
		GetWindowRect(hwnd, &r);

		if (0 == nIDDlgItem || ScreenToClientRect(hwndDlg, &r))
		{
			bResult = MoveWindow(hwnd, r.left + dx, r.top + dy, (r.right - r.left) + dw, (r.bottom - r.top) + dh, bRepaint);
		}
	}

	return(bResult);
}
*/

BOOL NLeoHelpers::EnableDlgItem(HWND hwndDlg, int nIDDlgItem, BOOL bEnable)
{
	BOOL bResult = FALSE;

	HWND hwnd = GetDlgItem(hwndDlg, nIDDlgItem);

	if (NULL != hwnd)
	{
		bResult = EnableWindow(hwnd, bEnable);
	}

	return(bResult);
}

BOOL NLeoHelpers::GetDlgItemText(HWND hwndDlg, int nIDDlgItem, std::basic_string< TCHAR > *pStrResult)
{
	BOOL bResult = FALSE;

	pStrResult->erase();

	HWND hwnd = GetDlgItem(hwndDlg, nIDDlgItem);

	if (NULL != hwnd)
	{
		int iLength = GetWindowTextLength(hwnd);

		if (0 == iLength)
		{
			bResult = TRUE;
		}
		else if (0 < iLength)
		{
			iLength++; // Not clear if GetWindowTextLength includes the null but GetWindowText does.

			TCHAR *szBuffer = new TCHAR[ iLength ];

			if (0 < GetWindowText(hwnd, szBuffer, iLength))
			{
				*pStrResult = szBuffer;

				bResult = TRUE;
			}

			delete [] szBuffer;
		}
	}

	return(bResult);
}

BOOL NLeoHelpers::GetListBoxItemText(HWND hwndListBox, LRESULT lrItemIndex, std::basic_string< TCHAR > *pStrResult)
{
	BOOL bResult = FALSE;

	pStrResult->clear();

	LRESULT itemTextLength = SendMessage(hwndListBox, LB_GETTEXTLEN, lrItemIndex, 0);
	if (0 == itemTextLength)
	{
		bResult = TRUE;
	}
	else if (0 < itemTextLength)
	{
		TCHAR *szItemText = new TCHAR[ itemTextLength + 1 ];

		if (0 < SendMessage(hwndListBox, LB_GETTEXT, lrItemIndex, reinterpret_cast<LPARAM>(szItemText)))
		{
			*pStrResult = szItemText;
			bResult = TRUE;
		}

		delete [] szItemText;
	}

	return(bResult);
}

/*
void NLeoHelpers::SubclassDlgItemToUserData(HWND hwndDlg, int nIDDlgItem, WNDPROC wndProc)
{
	HWND hwnd = GetDlgItem(hwndDlg, nIDDlgItem);

	if (NULL != hwnd)
	{
		SetWindowLongPtr(hwnd, GWLP_USERDATA, SetWindowLongPtr(hwnd, GWLP_WNDPROC, reinterpret_cast<LPARAM>(wndProc)));
	}
}
*/

BOOL NLeoHelpers::GetDlgItemLong(HWND hwndDlg, int nIDDlgItem, int nIndex, LONG *plResult)
{
	BOOL bResult = FALSE;

	HWND hwnd = GetDlgItem(hwndDlg, nIDDlgItem);

	if (NULL != hwnd)
	{
		SetLastError(0);

		LONG lStyleValue = GetWindowLong(hwnd, nIndex);

		if (0 != lStyleValue || 0 == GetLastError())
		{
			*plResult = lStyleValue;
			bResult = TRUE;
		}
	}

	return(bResult);
}

/*
BOOL NLeoHelpers::SetDlgItemLong(HWND hwndDlg, int nIDDlgItem, int nIndex, LONG lValue)
{
	BOOL bResult = FALSE;

	HWND hwnd = GetDlgItem(hwndDlg, nIDDlgItem);

	if (NULL != hwnd)
	{
		SetLastError(0);

		LONG lOldStyleValue = SetWindowLong(hwnd, nIndex, lValue);

		if (0 != lOldStyleValue || 0 == GetLastError())
		{
			bResult = TRUE;
		}
	}

	return(bResult);
}
*/
/*
BOOL NLeoHelpers::GetDlgItemLongPtr(HWND hwndDlg, int nIDDlgItem, int nIndex, LONG_PTR *plResult)
{
	BOOL bResult = FALSE;

	HWND hwnd = GetDlgItem(hwndDlg, nIDDlgItem);

	if (NULL != hwnd)
	{
		SetLastError(0);

		LONG_PTR lStyleValue = GetWindowLongPtr(hwnd, nIndex);

		if (0 != lStyleValue || 0 == GetLastError())
		{
			*plResult = lStyleValue;
			bResult = TRUE;
		}
	}

	return(bResult);
}
*/
/*
BOOL NLeoHelpers::SetDlgItemLongPtr(HWND hwndDlg, int nIDDlgItem, int nIndex, LONG_PTR lValue)
{
	BOOL bResult = FALSE;

	HWND hwnd = GetDlgItem(hwndDlg, nIDDlgItem);

	if (NULL != hwnd)
	{
		SetLastError(0);

		LONG_PTR lOldStyleValue = SetWindowLongPtr(hwnd, nIndex, lValue);

		if (0 != lOldStyleValue || 0 == GetLastError())
		{
			bResult = TRUE;
		}
	}

	return(bResult);
}
*/
/*
BOOL NLeoHelpers::ModifyDlgItemStyle(HWND hwndDlg, int nIDDlgItem, bool bStyleEx, LONG lAdd, LONG lRemove)
{
	BOOL bResult = FALSE;

	HWND hwnd = GetDlgItem(hwndDlg, nIDDlgItem);

	if (NULL != hwnd)
	{
		int nStyleKey = bStyleEx ? GWL_EXSTYLE : GWL_STYLE;

		SetLastError(0);

		LONG lStyleValue = GetWindowLong(hwnd, nStyleKey);

		if (0 != lStyleValue || 0 == GetLastError())
		{
			lStyleValue &= (~lRemove);
			lStyleValue |= lAdd;

			lStyleValue = SetWindowLong(hwnd, nStyleKey, lStyleValue);

			if (0 != lStyleValue || 0 == GetLastError())
			{
				bResult = TRUE;
			}
		}
	}

	return(bResult);
}
*/
/*
BOOL NLeoHelpers::InvalidateDlgItem(HWND hwndDlg, int nIDDlgItem)
{
	BOOL bResult = FALSE;

	HWND hwnd = GetDlgItem(hwndDlg, nIDDlgItem);

	if (NULL != hwnd)
	{
		bResult = InvalidateRect(hwnd, NULL, FALSE);
	}

	return(bResult);
}
*/

void NLeoHelpers::CenterWindow(HWND hWndToMove, HWND hWndRelative)
{
	bool bResult = true;

	RECT rectSnap;

	RECT rectRelative;

	if (NULL != hWndRelative)
	{
		::GetWindowRect(hWndRelative, &rectRelative);

		if (!NLeoHelpers::GetAreasFromWindow(NULL, &rectSnap, hWndRelative, MONITOR_DEFAULTTONEAREST))
		{
			bResult = false;
		}
	}
	else
	{
		// Get work area under the mouse pointer.

		POINT pointCursor;
		MONITORINFO monitorInfo;
		ZeroMemory(&monitorInfo, sizeof(monitorInfo));
		monitorInfo.cbSize = sizeof(monitorInfo);
		HMONITOR hMonitor;

		if (GetCursorPos(&pointCursor)
		&&	(NULL != (hMonitor = NLeoHelpers::MonitorFromPoint(pointCursor, MONITOR_DEFAULTTOPRIMARY)))
		&&	NLeoHelpers::GetMonitorInfo(hMonitor, &monitorInfo))
		{
			rectRelative = monitorInfo.rcWork;
			CopyRect(&rectSnap, &rectRelative);
		}
		else if (::GetWindowRect(::GetDesktopWindow(), &rectRelative))
		{
			CopyRect(&rectSnap, &rectRelative);
		}
		else
		{
			bResult = false;
		}
	}

	RECT rectToMove;

	if (bResult && ::GetWindowRect(hWndToMove, &rectToMove))
	{
		int relativeWidth  = (rectRelative.right  - rectRelative.left);
		int relativeHeight = (rectRelative.bottom - rectRelative.top);
		int toMoveWidth    = (rectToMove.right    - rectToMove.left);
		int toMoveHeight   = (rectToMove.bottom   - rectToMove.top);

		int x = rectRelative.left + ( (relativeWidth  - toMoveWidth ) / 2 );
		int y = rectRelative.top  + ( (relativeHeight - toMoveHeight) / 2 );

		if (x < rectSnap.left)
		{
			x = rectSnap.left;
		}

		if ((x + toMoveWidth) > rectSnap.right)
		{
			x = rectSnap.right - toMoveWidth;
		}

		if (y < rectSnap.top)
		{
			y = rectSnap.top;
		}

		if ((y + toMoveHeight) > rectSnap.bottom)
		{
			y = rectSnap.bottom - toMoveHeight;
		}

		::SetWindowPos(hWndToMove, NULL, x, y, 0, 0, SWP_NOSIZE|SWP_NOZORDER|SWP_NOACTIVATE);
	}
}

HMONITOR NLeoHelpers::MonitorFromPoint(POINT pt, DWORD dwFlags)
{
	HMONITOR hMonitorResult = NULL;

	HMODULE hModuleLibrary = LoadLibrary(USER32DLL_NAME);

	if (NULL != hModuleLibrary)
	{
		MONITORFROMPOINT_PTR funcMonitorFromPoint = reinterpret_cast<MONITORFROMPOINT_PTR>(GetProcAddress(hModuleLibrary, MONITORFROMPOINT_NAME));

		if (NULL != funcMonitorFromPoint)
		{
			hMonitorResult = funcMonitorFromPoint(pt, dwFlags);
		}

		DWORD dwError = GetLastError();
		FreeLibrary(hModuleLibrary);
		SetLastError(dwError);
	}

	return(hMonitorResult);
}

BOOL NLeoHelpers::GetMonitorInfo(HMONITOR hMonitor, LPMONITORINFO lpmi)
{
	BOOL bResult = FALSE;

	HMODULE hModuleLibrary = LoadLibrary(USER32DLL_NAME);

	if (NULL != hModuleLibrary)
	{
		GETMONITORINFO_PTR funcGetMonitorInfo = reinterpret_cast<GETMONITORINFO_PTR>(GetProcAddress(hModuleLibrary, GETMONITORINFO_NAME));

		if (NULL != funcGetMonitorInfo)
		{
			bResult = funcGetMonitorInfo(hMonitor, lpmi);
		}

		DWORD dwError = GetLastError();
		FreeLibrary(hModuleLibrary);
		SetLastError(dwError);
	}

	return(bResult);
}

bool NLeoHelpers::GetAreasFromWindow(RECT *pRectFull, RECT *pRectWork, HWND hWnd, DWORD dwFlags)
{
	if (NULL != pRectFull) { ZeroMemory(pRectFull, sizeof(RECT)); }
	if (NULL != pRectWork) { ZeroMemory(pRectWork, sizeof(RECT)); }

	bool bResult = false;

	bool bGetDefault = true;

	HMODULE hModuleLibrary = LoadLibrary(USER32DLL_NAME);

	if (NULL != hModuleLibrary)
	{
		MONITORFROMWINDOW_PTR funcMonitorFromWindow = reinterpret_cast<MONITORFROMWINDOW_PTR>(GetProcAddress(hModuleLibrary, MONITORFROMWINDOW_NAME));
		GETMONITORINFO_PTR    funcGetMonitorInfo    = reinterpret_cast<GETMONITORINFO_PTR   >(GetProcAddress(hModuleLibrary, GETMONITORINFO_NAME   ));

		if (NULL != funcMonitorFromWindow && NULL != funcGetMonitorInfo)
		{
			HMONITOR hMonitor = funcMonitorFromWindow(hWnd, dwFlags);

			if ((NULL == hMonitor) && (MONITOR_DEFAULTTONULL & dwFlags))
			{
				bGetDefault = false;
			}
			else if (NULL != hMonitor)
			{
				MONITORINFO mi;
				ZeroMemory(&mi, sizeof(mi));
				mi.cbSize = sizeof(mi);

				if (funcGetMonitorInfo(hMonitor, &mi)
				&&	(NULL == pRectFull || CopyRect(pRectFull, &mi.rcMonitor))
				&&	(NULL == pRectWork || CopyRect(pRectWork, &mi.rcWork   )))
				{
					bResult = true;
				}
			}
		}

		DWORD dwError = GetLastError();
		FreeLibrary(hModuleLibrary);
		SetLastError(dwError);
	}

	if (bGetDefault && (!bResult))
	{
		// These methods work on all platforms.
		// (On Win2k and above they will return the primary monitor's areas.)

		bResult = true;

		if (bResult && NULL != pRectFull)
		{
			pRectFull->left   = 0;
			pRectFull->top    = 0;
			pRectFull->right  = ::GetSystemMetrics(SM_CXFULLSCREEN);
			pRectFull->bottom = ::GetSystemMetrics(SM_CYFULLSCREEN);

			if (0 == pRectFull->right || 0 == pRectFull->bottom)
			{
				bResult = false;
			}
		}

		if (bResult && NULL != pRectWork)
		{
			if (!::SystemParametersInfo(SPI_GETWORKAREA, 0, pRectWork, 0))
			{
				bResult = false;
			}
		}
	}

	return(bResult);
}

HBITMAP NLeoHelpers::MakeBitmapWithAlpha(CxImage &xi, bool bMakeAllOpaque)
{
	HBITMAP hbmResult = NULL;

	const long lWidth = xi.GetWidth();
	const long lHeight = xi.GetHeight();

	BITMAPINFOHEADER bitmapHeader;
	bitmapHeader.biSize = sizeof(bitmapHeader);
	bitmapHeader.biWidth = lWidth;
	bitmapHeader.biHeight = -lHeight;
	bitmapHeader.biPlanes = 1;
	bitmapHeader.biBitCount = 32;
	bitmapHeader.biCompression = BI_RGB;
	bitmapHeader.biSizeImage = 0;
	bitmapHeader.biXPelsPerMeter = 0;
	bitmapHeader.biYPelsPerMeter = 0;
	bitmapHeader.biClrUsed = 0;
	bitmapHeader.biClrImportant = 0;

	HDC hMemDC = CreateCompatibleDC(NULL);
	if (NULL != hMemDC)
	{
		RGBQUAD *pBit32 = NULL;
		hbmResult = CreateDIBSection(hMemDC, reinterpret_cast<BITMAPINFO*>(&bitmapHeader), DIB_RGB_COLORS, reinterpret_cast<void**>(&pBit32), NULL, 0);
		GdiFlush();
		if (pBit32)
		{
			//RGBQUAD rgbWTF;
			//rgbWTF.rgbGreen = 100;
			//rgbWTF.rgbReserved = 10;

			for (long y = lHeight - 1; y >= 0; y--)
			{
				for (long x = 0; x < lWidth; x++)
				{
					*pBit32 = xi.GetPixelColor(x, y, true); // Includes alpha.
					if (bMakeAllOpaque) { pBit32->rgbReserved = 255; }
					pBit32++;
				}
			}
		}
		DeleteDC(hMemDC);
		}

	return(hbmResult);
}
